package edu.wpi.cs.jsh;

import java.util.*;

/**
 * Created by fred on 11/7/14.
 */
class BasicStream extends AbstractBaseStream implements Stream {

    private final Object m_lock;
    private final Map<String, NamedStream> m_subStreams = new HashMap<>();

    public BasicStream(String name, Object lock) {
        super(name, Stream.class);
        m_lock = lock;
    }

    @Override
    public <T> TypeStream<T> getTypeStream(String streamName, Class<T> streamType) throws JshException {
        synchronized (m_lock) {
            TypeStream<T> stream = getStream(streamName, streamType);
            if (stream == null) {
                stream = JshStreamFactory.getInstance().makeTypeStream(streamName, streamType, m_lock);
                m_subStreams.put(streamName, stream);
            }
            return stream;
        }
    }

    @Override
    public Stream getSubStream(String streamName) throws JshException {
        synchronized (m_lock) {
            Stream stream = getStream(streamName, Stream.class);
            if (stream == null) {
                stream = JshStreamFactory.getInstance().makeStream(streamName, m_lock, JshStreamFactory.StreamLevel.NORMAL);
                m_subStreams.put(streamName, stream);
            }
            return stream;
        }
    }

    @Override
    public Map<String, NamedStream> getSubStreams() {
        return m_subStreams;
    }

    private <S extends NamedStream, T extends Class<?>> S getStream(String name, T type) throws JshException {
        S stream = null;
        if (m_subStreams.containsKey(name)) {
            NamedStream mapStream = m_subStreams.get(name);
            if (mapStream.getStreamClass().equals(type)) {
                stream = (S) mapStream;
            } else {
                throw new JshException("Stream " + name + " exists already, and is a different type! Original was "
                        + mapStream.getStreamClass() + ", requested was " + type);
            }
        }
        return stream;
    }
}
