package edu.wpi.cs.jsh;

import java.util.Collection;
import java.util.Collections;
import java.util.LinkedList;

/**
 * Created by fred on 11/7/14.
 */
class BasicTypeStream<T> extends AbstractBaseStream implements TypeStream<T> {

    private final Object m_lock;
    private final Collection<T> m_objects;

    public BasicTypeStream(String name, Class<?> type, Object lock) {
        super(name, type);
        m_lock = lock;
        m_objects = new LinkedList<T>();
    }

    @Override
    public void output(T... values) {
        if (values != null) {
            synchronized (m_lock) {
                Collections.addAll(m_objects, values);
                try {
                    JshStreamController.getInstance().incrementCount(values.length);
                } catch (JshException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    /**
     * Gets the objects that have been buffered by this type stream
     *
     * @return The buffered objects
     */
    Collection<T> getObjects() {
        return m_objects;
    }

    /**
     * Removes all of the objects currently buffered.
     */
    void flushObjects() {
        m_objects.clear();
    }
}
