package edu.wpi.cs.jsh;

import javax.swing.plaf.basic.BasicTreeUI;

/**
 * This factory is internal to the jsh api. It is used to create new JshStreams at runtime.
 */
class JshStreamFactory {

    public static final JshStreamFactory instance = new JshStreamFactory();

    public static JshStreamFactory getInstance() {
        return instance;
    }

    /**
     * Creates a new instance of an {@link Stream} implementation with the given name. The type of the stream is
     * {@link Stream}
     *
     * @param name The name of the new stream
     * @return The stream instance
     */
    public Stream makeStream(String name, Object lock, StreamLevel level) throws JshException {
        Stream stream;
        switch (level) {
            case TOP_LEVEL:
                stream = new TopLevelStream(name, lock);
                break;
            case NORMAL:
                stream = new BasicStream(name, lock);
                break;
            default:
                throw new JshException("Error, unrecognized stream level");
        }
        return stream;
    }

    /**
     * Creates a new instace of an {@link TypeStream} implementation with the given name and base type. The type of
     * the stream is the provided type.
     *
     * @param name The name of the type stream
     * @param type The type of the stream. This is necessary because Java lacks reified generics
     * @param <T>  The type of the stream to be made.
     * @return The stream instance.
     */
    public <T> TypeStream<T> makeTypeStream(String name, Class<T> type, Object lock) {
        return new BasicTypeStream<T>(name, type, lock);
    }

    public enum StreamLevel {
        TOP_LEVEL, NORMAL
    }
}
