package edu.wpi.cs.jsh;

import com.google.gson.*;
import org.junit.Before;
import org.junit.Test;

import static org.junit.Assert.assertEquals;

/**
 * Created by fred on 11/10/14.
 */
public class BasicStreamTest {

    @Before
    public void setup() throws JshException {
        JshStreamController.reset();
        JshStreamController.initialize();
    }

    @Test
    public void testEmptySubStream() throws JshException {
        Stream superStream = JshStreamController.getInstance().StdOutStream;
        superStream.getSubStream("TestSubStream");
        Gson json = new GsonBuilder()
                .registerTypeAdapter(BasicTypeStream.class, new BasicTypeStreamSerializer())
                .registerTypeAdapter(BasicStream.class, new BasicStreamSerializer())
                .registerTypeAdapter(TopLevelStream.class, new TopLevelStreamSerializer())
                .create();
        JsonElement actual = json.toJsonTree(superStream);
        JsonObject expected = new JsonObject();
        JsonArray array = new JsonArray();
        array.add(new JsonObject());
        expected.add("TestSubStream", array);
        assertEquals(expected, actual);
    }

    @Test
    public void testIntSubStream() throws JshException {
        Stream superStream = JshStreamController.getInstance().StdOutStream;
        TypeStream<Integer> subStream = superStream.getTypeStream("TestSubStream", Integer.class);
        subStream.output(1, 2, 3);
        Gson json = new GsonBuilder()
                .registerTypeAdapter(BasicTypeStream.class, new BasicTypeStreamSerializer())
                .registerTypeAdapter(BasicStream.class, new BasicStreamSerializer())
                .registerTypeAdapter(TopLevelStream.class, new TopLevelStreamSerializer())
                .create();
        JsonElement actual = json.toJsonTree(superStream);
        JsonObject expected = new JsonObject();
        JsonArray array = new JsonArray();
        array.add(new JsonPrimitive(1));
        array.add(new JsonPrimitive(2));
        array.add(new JsonPrimitive(3));
        expected.add("TestSubStream", array);
        assertEquals(expected, actual);
    }

    @Test
    public void testStringSubStream() throws JshException {
        Stream superStream = JshStreamController.getInstance().StdOutStream;
        TypeStream<String> subStream = superStream.getTypeStream("TestSubStream", String.class);
        subStream.output("First", "Second", "Third");
        Gson json = new GsonBuilder()
                .registerTypeAdapter(BasicTypeStream.class, new BasicTypeStreamSerializer())
                .registerTypeAdapter(BasicStream.class, new BasicStreamSerializer())
                .registerTypeAdapter(TopLevelStream.class, new TopLevelStreamSerializer())
                .create();
        JsonElement actual = json.toJsonTree(superStream);
        JsonObject expected = new JsonObject();
        JsonArray array = new JsonArray();
        array.add(new JsonPrimitive("First"));
        array.add(new JsonPrimitive("Second"));
        array.add(new JsonPrimitive("Third"));
        expected.add("TestSubStream", array);
        assertEquals(expected, actual);
    }

    @Test
    public void testSubSubStream() throws JshException {
        Stream superStream = JshStreamController.getInstance().StdOutStream;
        superStream.getSubStream("SubStream").getTypeStream("SubSubStream", Integer.class);
        Gson json = new GsonBuilder()
                .registerTypeAdapter(BasicTypeStream.class, new BasicTypeStreamSerializer())
                .registerTypeAdapter(BasicStream.class, new BasicStreamSerializer())
                .registerTypeAdapter(TopLevelStream.class, new TopLevelStreamSerializer())
                .create();
        JsonElement actual = json.toJsonTree(superStream);
        JsonObject expected = new JsonObject();
        JsonArray array = new JsonArray();
        JsonObject subSubObject = new JsonObject();
        subSubObject.add("SubSubStream", new JsonArray());
        array.add(subSubObject);
        expected.add("SubStream", array);
        assertEquals(expected, actual);
    }
}
