package edu.wpi.cs.jsh;

import com.google.gson.*;
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.*;

/**
 * Testing basic type stream serialization and output
 */
public class BasicTypeStreamTest {

    @Before
    public void setup() {
        try {
            JshStreamController.reset();
            JshStreamController.initialize();
        } catch (JshException e) {
            e.printStackTrace();
        }
    }

    @Test
    public void testIntStream() throws JshException {
        TypeStream<Integer> intStream = JshStreamController.getInstance().StdOutStream.getTypeStream("IntStream", Integer.class);
        intStream.output(1, 2, 3);
        Gson json = new GsonBuilder().registerTypeAdapter(BasicTypeStream.class, new BasicTypeStreamSerializer()).create();
        JsonElement output = json.toJsonTree(intStream);
        JsonArray expected = new JsonArray();
        expected.add(new JsonPrimitive(1));
        expected.add(new JsonPrimitive(2));
        expected.add(new JsonPrimitive(3));
        assertEquals(output, expected);
    }

    @Test
    public void testStringStream() throws JshException {
        TypeStream<String> intStream = JshStreamController.getInstance().StdOutStream.getTypeStream("IntStream", String.class);
        intStream.output("First", "Second", "Third");
        Gson json = new GsonBuilder().registerTypeAdapter(BasicTypeStream.class, new BasicTypeStreamSerializer()).create();
        JsonElement output = json.toJsonTree(intStream);
        JsonArray expected = new JsonArray();
        expected.add(new JsonPrimitive("First"));
        expected.add(new JsonPrimitive("Second"));
        expected.add(new JsonPrimitive("Third"));
        assertEquals(output, expected);
    }

    @Test
    public void testObjectStream() throws JshException {
        TypeStream<TestClass> intStream = JshStreamController.getInstance().StdOutStream.getTypeStream("IntStream", TestClass.class);
        TestClass firstElement = new TestClass(1 , 2, "three");
        TestClass secondElement = new TestClass(4, 5, "six");
        TestClass thirdElement = new TestClass(7, 8, "nine");
        intStream.output(firstElement, secondElement, thirdElement);
        Gson json = new GsonBuilder().registerTypeAdapter(BasicTypeStream.class, new BasicTypeStreamSerializer()).create();
        JsonElement output = json.toJsonTree(intStream);
        JsonArray expected = new JsonArray();
        expected.add(json.toJsonTree(firstElement));
        expected.add(json.toJsonTree(secondElement));
        expected.add(json.toJsonTree(thirdElement));
        assertEquals(output, expected);
    }

    private class TestClass {
        int value1;
        int value2;
        String value3;

        public TestClass(int value1, int value2, String value3) {
            this.value1 = value1;
            this.value2 = value2;
            this.value3 = value3;
        }
    }
}
